from __future__ import annotations

from typing import TYPE_CHECKING

from loguru import logger

from discord_reminder_bot.parsers import calculate, parse_time
from discord_reminder_bot.settings import scheduler

if TYPE_CHECKING:
    import datetime

    import discord
    from apscheduler.job import Job
    from discord.interactions import InteractionChannel


async def add_reminder_job(
    interaction: discord.Interaction,
    message: str,
    time: str,
    channel: discord.TextChannel | None = None,
    user: discord.User | None = None,
    dm_and_current_channel: bool | None = None,
) -> None:
    """Adds a reminder job to the scheduler based on user input.

    Schedules a message to be sent at a specified time either to a specific channel,
    a specific user via direct message, or both. It handles permission checks,
    time parsing, and job creation using the APScheduler instance.

    Args:
        interaction: The discord interaction object initiated by the user command.
        message: The content of the reminder message to be sent.
        time: A string representing the date and time for the reminder.
            This string will be parsed to a datetime object.
        channel: The target text channel for the reminder. If None, defaults
            to the channel where the interaction occurred.
        user: The target user to send a direct message reminder to. If None,
            no direct message is scheduled.
        dm_and_current_channel: If True and a user is specified, sends the
            reminder to both the user's DM and the target channel. If False
            and a user is specified, only sends the DM. Defaults to None,
            behaving like False if only a user is specified, or sending only
            to the channel if no user is specified.

    """
    logger.info(f"New reminder from {interaction.user} ({interaction.user.id}) in {interaction.channel}")
    logger.info(f"Arguments: {locals()}")

    # Check if we have access to the specified channel or the current channel
    target_channel: InteractionChannel | discord.TextChannel | None = channel or interaction.channel
    if target_channel and interaction.guild and not target_channel.permissions_for(interaction.guild.me).send_messages:
        await interaction.followup.send(content=f"I don't have permission to send messages in <#{target_channel.id}>.", ephemeral=True)

    # Get the channel ID
    channel_id: int | None = channel.id if channel else (interaction.channel.id if interaction.channel else None)
    if not channel_id:
        await interaction.followup.send(content="Failed to get channel.", ephemeral=True)
        return

    # Ensure the guild is valid
    guild: discord.Guild | None = interaction.guild or None
    if not guild:
        await interaction.followup.send(content="Failed to get guild.", ephemeral=True)
        return

    dm_message: str = ""
    if user:
        parsed_time: datetime.datetime | None = parse_time(date_to_parse=time)
        if not parsed_time:
            await interaction.followup.send(content=f"Failed to parse time: {time}.", ephemeral=True)
            return

        user_reminder: Job = scheduler.add_job(
            func="discord_reminder_bot.main:send_to_user",
            trigger="date",
            run_date=parsed_time,
            kwargs={
                "user_id": user.id,
                "guild_id": guild.id,
                "message": message,
            },
        )
        logger.info(f"User reminder job created: {user_reminder} for {user.id} at {parsed_time}")

        dm_message = f" and a DM to {user.display_name}"
        if not dm_and_current_channel:
            msg = (
                f"Hello {interaction.user.display_name},\n"
                f"I will send a DM to {user.display_name} at:\n"
                f"First run in {calculate(user_reminder)} with the message:\n**{message}**."
            )
            await interaction.followup.send(content=msg)
            return

    # Create channel reminder job
    channel_job: Job = scheduler.add_job(
        func="discord_reminder_bot.main:send_to_channel",
        trigger="date",
        run_date=parse_time(date_to_parse=time),
        kwargs={
            "channel_id": channel_id,
            "message": message,
            "author_id": interaction.user.id,
        },
    )
    logger.info(f"Channel reminder job created: {channel_job} for {channel_id}")

    msg: str = (
        f"Hello {interaction.user.display_name},\n"
        f"I will notify you in <#{channel_id}>{dm_message}.\n"
        f"First run in {calculate(channel_job)} with the message:\n**{message}**."
    )

    await interaction.followup.send(content=msg)
